=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2006-01-05 10:49:26 +0000
+++ bzrlib/symbol_versioning.py	2006-01-05 11:03:07 +0000
@@ -37,6 +37,28 @@
     warn = method
 
 
+# TODO - maybe this would be easier to use as one 'smart' method that
+# guess if it is a method or a class or an attribute ? If so, we can
+# add that on top of the primitives, once we have all three written
+# - RBC 20050105
+
+def deprecated_function(deprecation_version):
+    """Decorate a function so that use of it will trigger a warning."""
+
+    def function_decorator(callable):
+        """This is the function python calls to perform the decoration."""
+        
+        def decorated_function(*args, **kwargs):
+            """This is the decorated method."""
+            symbol = "%s.%s" % (callable.__module__, 
+                                callable.__name__
+                                )
+            warn(deprecation_version % symbol, DeprecationWarning)
+            return callable(*args, **kwargs)
+        return decorated_function
+    return function_decorator
+
+
 def deprecated_method(deprecation_version):
     """Decorate a method so that use of it will trigger a warning.
     

=== modified file 'bzrlib/tests/test_symbol_versioning.py'
--- bzrlib/tests/test_symbol_versioning.py	2006-01-05 10:49:26 +0000
+++ bzrlib/tests/test_symbol_versioning.py	2006-01-05 11:03:07 +0000
@@ -21,6 +21,10 @@
 from bzrlib.tests import TestCase
 
 
+@symbol_versioning.deprecated_function(symbol_versioning.zero_seven)
+def deprecated_function():
+    return 1
+
 
 class TestDeprecationWarnings(TestCase):
 
@@ -33,18 +37,30 @@
     
     @symbol_versioning.deprecated_method(symbol_versioning.zero_seven)
     def deprecated_method(self):
-        return
+        return 1
 
     def test_deprecated_method(self):
+        expected_warning = (
+            "bzrlib.tests.test_symbol_versioning."
+            "TestDeprecationWarnings.deprecated_method "
+            "was deprecated in version 0.7", DeprecationWarning)
+        self.check_deprecated_callable(expected_warning,
+                                       self.deprecated_method)
+
+    def test_deprecated_function(self):
+        expected_warning = (
+            "bzrlib.tests.test_symbol_versioning.deprecated_function "
+            "was deprecated in version 0.7", DeprecationWarning)
+        self.check_deprecated_callable(expected_warning,
+                                       deprecated_function)
+
+    def check_deprecated_callable(self, expected_warning, deprecated_callable):
         old_warning_method = symbol_versioning.warn
-        expected_warning = (
-            "bzrlib.tests.test_symbol_versioning.TestDeprecationWarnings.deprecated_method "
-            "was deprecated in version 0.7", DeprecationWarning)
         try:
             symbol_versioning.set_warning_method(self.capture_warning)
-            self.deprecated_method()
+            self.assertEqual(1, deprecated_callable())
             self.assertEqual([expected_warning], self._warnings)
-            self.deprecated_method()
+            deprecated_callable()
             self.assertEqual([expected_warning, expected_warning],
                              self._warnings)
         finally:

